@@ -59,7 +59,7 @@ class JSONDecoder(json.JSONDecoder):
 
 class Session(object):
 
-	__version__ = "1.2.2-4"
+	__version__ = "1.2.4-2"
 
 	def __init__(self, domain=None, username=None, api_key=None, pwd=None, hostname=None, port=None, proxy=None, verbose=False, dev=False):
 		''' Setup; store credentials, authenticate, get a session key '''
@@ -70,16 +70,16 @@ class Session(object):
 		self._proxy = proxy
 		self._dev = dev
 		self._clearance = CLEARANCE_NONE
-		self.verbose("Creating AccSyn Python API session (v%s)"%Session.__version__)
+		self.verbose("Creating Accsyn Python API session (v%s)"%Session.__version__)
 		# Migrate
 		for key in os.environ:
 			if key.startswith("FILMHUB_"):
 				Session.warning("Found old FilmHUB product environment variable '%s', please migrate!"%key)
 		if domain is None:
-			assert ('ACCSYN_DOMAIN' in os.environ or 'ACCSYN_ORG' in os.environ or 'FILMUB_DOMAIN' in os.environ or 'FILMHUB_ORG' in os.environ),("Please supply your AccSyn domain/organization or set ACCSYN_DOMAIN environment!")
-		self._domain = domain or (os.environ['ACCSYN_DOMAIN'] if 'ACCSYN_DOMAIN' in os.environ else os.environ.get('ACCSYN_ORG', os.environ.get('FILMHUB_DOMAIN'), os.environ.get('FILMHUB_ORG')))
+			assert ('ACCSYN_DOMAIN' in os.environ or 'ACCSYN_ORG' in os.environ or 'FILMUB_DOMAIN' in os.environ or 'FILMHUB_ORG' in os.environ),("Please supply your Accsyn domain/organization or set ACCSYN_DOMAIN environment!")
+		self._domain = domain or (os.environ['ACCSYN_DOMAIN'] if 'ACCSYN_DOMAIN' in os.environ else os.environ.get('ACCSYN_ORG', os.environ.get('FILMHUB_DOMAIN', os.environ.get('FILMHUB_ORG'))))
 		if username is None:
-			assert ('ACCSYN_API_USER' in os.environ or 'FILMHUB_API_USER' in os.environ),("Please supply your AccSyn user name (E-mail) or set ACCSYN_API_USER environment!")
+			assert ('ACCSYN_API_USER' in os.environ or 'FILMHUB_API_USER' in os.environ),("Please supply your Accsyn user name (E-mail) or set ACCSYN_API_USER environment!")
 		self._username = username or os.environ.get('ACCSYN_API_USER') or os.environ['FILMHUB_API_USER']
 		if api_key:
 			self._api_key = api_key
@@ -90,7 +90,7 @@ class Session(object):
 				# Store it temporarily
 				self._pwd = pwd
 			else:
-				raise Exception("Please supply your AccSyn API KEY or set ACCSYN_API_KEY environment!")
+				raise Exception("Please supply your Accsyn API KEY or set ACCSYN_API_KEY environment!")
 		self._hostname = hostname
 		self._port = port or ACCSYN_PORT
 		if self._hostname is None:
@@ -208,7 +208,7 @@ class Session(object):
 	# REST get
 
 	def event(self, method, uri, data, query=None, entityid=None, timeout=None, ssl=True, quiet=False):
-		''' Generate an Event and send using REST to AccSyn cloud '''
+		''' Generate an Event and send using REST to Accsyn cloud '''
 		assert (self._session_key),("Please login before attempting to post event!")
 		event = {
 			'audience':"api",
@@ -260,25 +260,52 @@ class Session(object):
 		return retval
 
 	def rest(self, method, hostname, uri, data, timeout=None, ssl=True, port=None, quiet=False, headers=None):
-		''' Talk REST with FIlmHUB Cloud '''
+		''' Talk REST with Accsyn Cloud '''
 		if port is None:
 			port = self._port or ACCSYN_PORT
 		if hostname is None:
 			hostname = "%s.%s"%(self._domain, ACCSYN_CLOUD_DOMAIN)
 		# Proxy set?
-		proxy = self._proxy or os.environ.get('ACCSYN_PROXY')
-		if 0<len(proxy or ""):
-			if 0<proxy.find(":"):
-				parts = proxy.split(":")
-				hostname = parts[0]
-				port = int(parts[1])
+		proxy_type = None
+		proxy_hostname = None
+		proxy_port = -1
+		proxy_setting = self._proxy or os.environ.get('ACCSYN_PROXY')
+		if 0<len(proxy_setting or ""):
+			if 0<proxy_setting.find(":"):
+				parts = proxy_setting.split(":")
+				if len(parts) == 3:
+					proxy_type = parts[0]
+					proxy_hostname = parts[1]
+					proxy_port = int(parts[2])
+				elif len(parts) == 2:
+					proxy_type = "accsyn"
+					proxy_hostname = parts[0]
+					proxy_port = int(parts[1])
 			else:
-				hostname = proxy
-				port = 80
+				proxy_type = "accsyn"
+				proxy_hostname = proxy
+				proxy_port = 80
 			ssl = False
-		elif self._dev and (uri or "").find("registry") != 0:
+		if self._dev and (uri or "").find("registry") != 0:
 			ssl = False
 			port = 80
+		if proxy_type == "accsyn":
+			if proxy_port == -1:
+				proxy_port = 80
+			self.verbose("Using Accsyn proxy @ %s:%s"%(proxy_hostname, proxy_port))
+			hostname = proxy_hostname
+			port = proxy_port
+		elif proxy_type in ["socks","socks5"]:
+			try:
+				self.verbose("Using SOCKS5 proxy @ %s:%s"%(proxy_hostname, proxy_port))
+				import socks
+				socks.setdefaultproxy(socks.PROXY_TYPE_SOCKS5, proxy_hostname, proxy_port)
+				socket.socket = socks.socksocket
+			except ImportError, ie:
+				Session.warning("Python lacks SOCKS support, please install 'pysocks' and try again...")
+				raise ie
+		elif not proxy_type is None:
+			raise Exception("Unknown proxy type '%s'!"%proxy_type)
 		url = "http%s://%s:%d/api/v1.0%s"%("s" if ssl else "", hostname, port,("/" if not uri.startswith("/") else "")+uri)
 		if timeout is None:
 			timeout = 999999
@@ -288,11 +315,11 @@ class Session(object):
 		#r = requests.post(github_url, data, auth=('user', '*****'))
 		CONNECT_TO, READ_TO = (10, 2*60)  # Wait 10s to reach machine, 2min for it to send back data
 		r = None
-		#data = AccSyn.prepare_rest_serialize(data)
+		#data = Accsyn.prepare_rest_serialize(data)
 		initial_timeout = timeout
 		if headers is None:
 			if uri.find("registry/")!=0:
-				assert (not self._session_key is None),("Need to be authenticated when communicating with AccSyn!")
+				assert (not self._session_key is None),("Need to be authenticated when communicating with Accsyn!")
 				headers = {'Authorization':"ASSession %s"%(base64.b64encode('{"domain":"%s","username":"%s","session_key":"%s"}'%(self._domain, self._username, self._session_key)))}
 			else:
 				headers = {}
@@ -324,7 +351,7 @@ class Session(object):
 				time.sleep(sleep_time)
 
 			try:
-				#retval = AccSyn.prepare_rest_deserialize(r.json())
+				#retval = Accsyn.prepare_rest_deserialize(r.json())
 				retval = json.loads(r.text, cls=JSONDecoder)
 				if not quiet:
 					self.verbose("%s/%s REST %s result of %s: %s (~%sms)"%(
@@ -527,9 +554,20 @@ class Session(object):
 		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, unicode))),("Invalid entity type supplied, must be of string type!")
 		assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, unicode))),("Invalid entity ID supplied, must be of string type!")
 		assert (0<len(data or {}) and isinstance(data, dict)),("Invalid data supplied, must be dict and have content!")
-		d = self.event("PUT", "%s/edit"%Session.get_base_uri(entitytype), data, entityid=entityid)
-		if d:
-			return d['result']
+		response = self.event("PUT", "%s/edit"%Session.get_base_uri(entitytype), data, entityid=entityid)
+		if response:
+			return response['result']
+
+	def update_many(self, entitytype, data, entityid=None):
+		''' Update many entities '''
+		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, str))),("Invalid entity type supplied, must be of string type!")
+		assert (entitytype.lower() == "task"),("Only multiple 'task' entities can be updated!")		
+		if entitytype.lower() == "task":
+			assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, str))),("Invalid entity ID supplied, must be of string type!")
+		assert (0<len(data or []) and isinstance(data, list)),("Invalid data supplied, must be a list!")
+		response = self.event("PUT", "%s/edit"%Session.get_base_uri(entitytype), data, entityid=entityid)
+		if response:
+			return response['result']
 
 	# Delete an entity
 
@@ -537,9 +575,9 @@ class Session(object):
 		''' Update an entity '''
 		assert (0<len(entitytype or "") and (isinstance(entitytype, str) or isinstance(entitytype, unicode))),("Invalid entity type supplied, must be of string type!")
 		assert (0<len(entityid or "") and (isinstance(entityid, str) or isinstance(entityid, unicode))),("Invalid entity ID supplied, must be of string type!")
-		d = self.event("DELETE", "%s/delete"%Session.get_base_uri(entitytype), {}, entityid=entityid)
-		if d:
-			return d['result'][0]
+		response = self.event("DELETE", "%s/delete"%Session.get_base_uri(entitytype), {}, entityid=entityid)
+		if response:
+			return response['result'][0]
 
 	# File operations
 	def ls(self, p, recursive=False, maxdepth=None, files_only=False, directories_only=False):
@@ -556,9 +594,9 @@ class Session(object):
 			data['directories_only'] = directories_only
 		if files_only:
 			data['files_only'] = files_only
-		d = self.event("GET", "organization/file", data)
-		if d:
-			return d['result']
+		response = self.event("GET", "organization/file", data)
+		if response:
+			return response['result']
 
 	def getsize(self, p):
 		assert (0<len(p or "") and (isinstance(p, str) or isinstance(p, unicode) or isinstance(p, dict) or isinstance(p, list))),("No path supplied, or not a string/list!")
@@ -566,9 +604,9 @@ class Session(object):
 			'op':"getsize",
 			'path':p,
 		}
-		d = self.event("GET", "organization/file", data)
-		if d:
-			return d['result']
+		response = self.event("GET", "organization/file", data)
+		if response:
+			return response['result']
 
 	def exists(self, p):
 		assert (0<len(p or "") and (isinstance(p, str) or isinstance(p, unicode) or isinstance(p, dict) or isinstance(p, list))),("No path supplied, or not a string/list!")
@@ -576,9 +614,28 @@ class Session(object):
 			'op':"exists",
 			'path':p,
 		}
-		d = self.event("GET", "organization/file", data)
-		if d:
-			return d['result']
+		response = self.event("GET", "organization/file", data)
+		if response:
+			return response['result']
+
+	# Pre publish
+	def prepublish(self, data):
+		if data is None or not isinstance(data, list):
+			raise Exception("None or empty data supplied!")
+		# Check entries, calculate size
+		def recursive_get_size(files):
+			result = 0
+			for d in files:
+				if not 'size' in d:
+					d['size'] = 0 if not d.get('is_dir') is True or d.get('files') is None else recursive_get_size(d['files'])
+				result += d['size']
+			return result
+		event_data = {
+			'files':data,
+			'size':recursive_get_size(data)
+		}
+		response = self.event("PUT", "organization/publish/preprocess", event_data)
+		return response['result']
 
 	# Misc
 	def get_api_key(self):
